@@ -1,7 +1,6 @@
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE RankNTypes #-}
-{-# LANGUAGE TypeApplications #-}
 
 -----------------------------------------------------------------------------
 
@@ -36,7 +35,6 @@ module Distribution.Simple.PreProcess
   , runSimplePreProcessor
   , ppCpp
   , ppCpp'
-  , ppGreenCard
   , ppC2hs
   , ppHsc2hs
   , ppHappy
@@ -347,8 +345,8 @@ preprocessFile mbWorkDir searchLoc buildLoc forSDist baseFile verbosity builtinS
           createDirectoryIfMissingVerbose verbosity True destDir
           runPreProcessorWithHsBootHack
             pp
-            (getSymbolicPath $ psrcLoc, getSymbolicPath $ psrcRelFile)
-            (getSymbolicPath $ buildLoc, srcStem <.> "hs")
+            (psrcLoc, getSymbolicPath $ psrcRelFile)
+            (buildLoc, srcStem <.> "hs")
   where
     i = interpretSymbolicPath mbWorkDir -- See Note [Symbolic paths] in Distribution.Utils.Path
     buildAsSrcLoc :: SymbolicPath Pkg (Dir Source)
@@ -363,20 +361,25 @@ preprocessFile mbWorkDir searchLoc buildLoc forSDist baseFile verbosity builtinS
       pp
       (inBaseDir, inRelativeFile)
       (outBaseDir, outRelativeFile) = do
+        -- Preprocessors are expected to take into account the working
+        -- directory, e.g. using runProgramCwd with a working directory
+        -- computed with mbWorkDirLBI.
+        -- Hence the use of 'getSymbolicPath' here.
         runPreProcessor
           pp
-          (inBaseDir, inRelativeFile)
-          (outBaseDir, outRelativeFile)
+          (getSymbolicPath $ inBaseDir, inRelativeFile)
+          (getSymbolicPath $ outBaseDir, outRelativeFile)
           verbosity
 
-        exists <- doesFileExist inBoot
-        when exists $ copyFileVerbose verbosity inBoot outBoot
-        where
+        -- Here we interact directly with the file system, so we must
+        -- interpret symbolic paths with respect to the working directory.
+        let
+          inFile = normalise (i inBaseDir </> inRelativeFile)
+          outFile = normalise (i outBaseDir </> outRelativeFile)
           inBoot = replaceExtension inFile "hs-boot"
           outBoot = replaceExtension outFile "hs-boot"
-
-          inFile = normalise (inBaseDir </> inRelativeFile)
-          outFile = normalise (outBaseDir </> outRelativeFile)
+        exists <- doesFileExist inBoot
+        when exists $ copyFileVerbose verbosity inBoot outBoot
 
 -- ------------------------------------------------------------
 
@@ -384,20 +387,6 @@ preprocessFile mbWorkDir searchLoc buildLoc forSDist baseFile verbosity builtinS
 
 -- ------------------------------------------------------------
 
-ppGreenCard :: BuildInfo -> LocalBuildInfo -> ComponentLocalBuildInfo -> PreProcessor
-ppGreenCard _ lbi _ =
-  PreProcessor
-    { platformIndependent = False
-    , ppOrdering = unsorted
-    , runPreProcessor = mkSimplePreProcessor $ \inFile outFile verbosity ->
-        runDbProgramCwd
-          verbosity
-          (mbWorkDirLBI lbi)
-          greencardProgram
-          (withPrograms lbi)
-          (["-tffi", "-o" ++ outFile, inFile])
-    }
-
 -- This one is useful for preprocessors that can't handle literate source.
 -- We also need a way to chain preprocessors.
 ppUnlit :: PreProcessor
@@ -511,8 +500,6 @@ ppHsc2hs bi lbi clbi =
             withResponseFile
               verbosity
               defaultTempFileOptions
-              mbWorkDir
-              (makeSymbolicPath $ takeDirectory outFile)
               "hsc2hs-response.txt"
               Nothing
               pureArgs
@@ -763,10 +750,6 @@ platformDefines lbi =
         ++ ["-D" ++ arch ++ "_BUILD_ARCH=1"]
         ++ map (\os' -> "-D" ++ os' ++ "_HOST_OS=1") osStr
         ++ map (\arch' -> "-D" ++ arch' ++ "_HOST_ARCH=1") archStr
-    HaskellSuite{} ->
-      ["-D__HASKELL_SUITE__"]
-        ++ map (\os' -> "-D" ++ os' ++ "_HOST_OS=1") osStr
-        ++ map (\arch' -> "-D" ++ arch' ++ "_HOST_ARCH=1") archStr
     _ -> []
   where
     comp = compiler lbi
@@ -876,11 +859,10 @@ standardPP lbi prog args =
 ppSuffixes :: [PPSuffixHandler] -> [Suffix]
 ppSuffixes = map fst
 
--- | Standard preprocessors: GreenCard, c2hs, hsc2hs, happy, alex and cpphs.
+-- | Standard preprocessors: c2hs, hsc2hs, happy, alex and cpphs.
 knownSuffixHandlers :: [PPSuffixHandler]
 knownSuffixHandlers =
-  [ (Suffix "gc", ppGreenCard)
-  , (Suffix "chs", ppC2hs)
+  [ (Suffix "chs", ppC2hs)
   , (Suffix "hsc", ppHsc2hs)
   , (Suffix "x", ppAlex)
   , (Suffix "y", ppHappy)
