@@ -438,13 +438,6 @@ configureOptions showOrParseArgs =
             [ (Flag GHC, ("g", ["ghc"]), "compile with GHC")
             , (Flag GHCJS, ([], ["ghcjs"]), "compile with GHCJS")
             , (Flag UHC, ([], ["uhc"]), "compile with UHC")
-            , -- "haskell-suite" compiler id string will be replaced
-              -- by a more specific one during the configure stage
-
-              ( Flag (HaskellSuite "haskell-suite")
-              , ([], ["haskell-suite"])
-              , "compile with a haskell-suite compiler"
-              )
             ]
         )
     , option
@@ -1101,7 +1094,7 @@ configureArgs bcHack flags =
       (Flag hc, NoFlag) -> [hc_flag_name ++ prettyShow hc]
       (NoFlag, NoFlag) -> []
     hc_flag_name
-      -- TODO kill off thic bc hack when defaultUserHooks is removed.
+      -- TODO kill off this bc hack when defaultUserHooks is removed.
       | bcHack = "--with-hc="
       | otherwise = "--with-compiler="
     optFlag name config_field = case config_field flags of
