@@ -111,6 +111,9 @@ import Distribution.Simple.Utils as Utils
 import Distribution.System
   ( Platform
   )
+import Distribution.Types.DependencySatisfaction
+  ( DependencySatisfaction (..)
+  )
 import Distribution.Types.GivenComponent
   ( GivenComponent (..)
   )
@@ -296,6 +299,7 @@ configureSetupScript
       , useDependenciesExclusive = not defaultSetupDeps && isJust explicitSetupDeps
       , useVersionMacros = not defaultSetupDeps && isJust explicitSetupDeps
       , isInteractive = False
+      , isMainLibOrExeComponent = True
       }
     where
       -- When we are compiling a legacy setup script without an explicit
@@ -550,7 +554,7 @@ configurePackage
       pkg = case finalizePD
         flags
         (enableStanzas stanzas)
-        (const True)
+        (const Satisfied)
         platform
         comp
         []
